/*
   FNT - Stream handler for socket connections
   Some network function for handle connections

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"
#include "../include/plugin.h"
#include "../include/stream.h"
#include "../include/mainloop.h"

#include "main.h"
#include "utils.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

bool
fnt_stream_socketPton (char *src, struct sockaddr_storage * dst)
{
#ifndef HAVE_INET_PTON
    struct sockaddr_in *sin;

    memset (dst, 0, sizeof (struct sockaddr_storage));
    sin = (struct sockaddr_in *) dst;

    if (inet_aton (src, &sin->sin_addr))
    {
	dst->ss_family = AF_INET;
	return true;
    }

    return false;
#else
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;

    memset (dst, 0, sizeof (struct sockaddr_storage));
    sin = (struct sockaddr_in *) dst;
    sin6 = (struct sockaddr_in6 *) dst;

    if (inet_pton (AF_INET, src, &sin->sin_addr) > 0)
    {
	dst->ss_family = AF_INET;
	return true;
    }

    if (inet_pton (AF_INET6, src, &sin6->sin6_addr) > 0)
    {
	dst->ss_family = AF_INET6;
#ifdef SIN6_LEN
	sin6->sin6_len = sizeof (struct sockaddr_in6);
#endif
	return true;
    }
    return true;
#endif
}

bool
fnt_stream_socketSetPort (struct sockaddr_storage * sa, in_port_t port)
{
    struct sockaddr_in *sin;
    struct sockaddr_in6 *sin6;

    sin = (struct sockaddr_in *) sa;
    sin6 = (struct sockaddr_in6 *) sa;

    switch (sa->ss_family)
    {
    case AF_INET:
	sin->sin_port = htons (port);
	return true;
    case AF_INET6:
	sin6->sin6_port = htons (port);
	return true;
    default:
	return false;
    }
}

socklen_t
fnt_stream_socketAddrLen (struct sockaddr_storage * sa)
{
#ifdef SIN6_LEN
    if (sa->ss_len != 0)
	return sa->ss_len;
#endif
    switch (sa->ss_family)
    {
    case AF_INET:
	return sizeof (struct sockaddr_in);
    case AF_INET6:
	return sizeof (struct sockaddr_in6);
    default:
	return sizeof (struct sockaddr_storage);
    }
}


bool
fnt_stream_parceSocketInfo (__fnt_stream_socketInfo_t * socketInfo,
			     const char *data)
{
    char *tmp_data = (char *) data;
    int data_len, tmp_seek = 0;

    if (data == NULL)
	return false;

    if (g_ascii_strncasecmp (tmp_data, "server:", 7) == 0)
	socketInfo->is_server = true;
    else if (g_ascii_strncasecmp (tmp_data, "client:", 7) == 0)
	socketInfo->is_server = false;
    else
    {
	fnt_error_push (FNT_MODULE_stream, FNT_STREAM_E_PARCE,
			_("Unknown socket type"));
	return false;
    }

    tmp_data += 7;
    data_len = strlen (tmp_data);
    while (tmp_data[tmp_seek])
    {
	if (tmp_data[tmp_seek] == ':')
	    break;
	tmp_seek++;
    }
    if (tmp_data[tmp_seek] == '\0')
    {
	fnt_error_push (FNT_MODULE_stream, FNT_STREAM_E_PARCE,
			_("Port number unspecified"));
	return false;
    }
    socketInfo->host = g_strndup (tmp_data, tmp_seek);
    tmp_data += tmp_seek + 1;
    socketInfo->port = atoi (tmp_data);
    return true;
}
